package org.ksv.plugin;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.FileStoreEditorInput;
import org.eclipse.ui.ide.IDE;

import com.openedge.pdt.core.OECommonServicesPlugin;
import com.openedge.pdt.text.OEProject;
import com.openedge.pdt.text.project.ProjectPropath;
import com.openedge.pdt.text.project.PropathEntry;

/**
 * Utility class with necessary plugin functions
 * @author ksv
 *
 */
public class KsvUtils {
	/**
	 * Gets current OpenEdge project using either the selection or
	 * the file opened in the active editor
	 * @param selection 
	 * @param window 
	 * @return
	 */
	@SuppressWarnings("deprecation")
	public static OEProject getActiveOEProject()
	{
		IProject project = getActiveProject();		
		
		if(project == null)
			return null;
		else
			return OEProject.getOEProject(project);
	}
	
	/**
	 * Gets current project using either the selection or
	 * the file opened in the active editor
	 * @return
	 */
	public static IProject getActiveProject() {
		IResource resource = null;

		resource = OECommonServicesPlugin.getSelectedResource();

		if (resource == null)
			resource = OECommonServicesPlugin.getEditorResource();


		if(resource == null)
			return null;
		else
			return resource.getProject();
		
	}
	
	
	/**
	 * Opens the common file dialog to select a file
	 * @param window       	- current window
	 * @param title			- title of the dialog
	 * @param fileFilters	- file filters
	 * @param filename		- initial filename
	 * @return filename or null if a user clicked cancel
	 */
	public static String openFileDialog(IWorkbenchWindow window,String title,String[] fileFilters, String filename) {
		FileDialog dialog = new FileDialog(window.getShell());
		dialog.setFilterExtensions(fileFilters);
		dialog.setText(title);
		dialog.setFileName(filename);
		return dialog.open();
	}	
	
	/**
	 * Opens the common folder dialog to select a folder
	 * @param window - current window
	 * @param title  - title of the dialog
	 * @return a folder or null if a user clicked cancel
	 */
	public static String openFolderDialog ( IWorkbenchWindow window, String title ) {
		DirectoryDialog dialog = new DirectoryDialog( window.getShell() );
		dialog.setText( title );
		return dialog.open( );		
	}
	
	/**
	 * Opens an external or internal file in the editor
	 * @param fileToOpen - file to open	 * 
	 * @return true - if file's been opened successfully
	 */
	public static boolean openFileInEditor( File fileToOpen ) {
					
		// Check whether the file we want to open is in the project.
		// If it's not, open it in read-only mode
		boolean readonly = lastProject.file2resource( fileToOpen.getAbsolutePath() ) == null;
		
	    try {	
	    	
	    	// There's no simple way to make the editor readonly, so we make a copy of the file readonly
	    	if ( readonly ) {
	    		try {
	    			IFileStore srcFile = EFS.getStore( fileToOpen.toURI() );
	    			// Form a new file name in the temp directory
	    			IPath newPath = Activator.getDefault().getStateLocation().addTrailingSeparator().append( fileToOpen.getName() );
		    		IFileStore tmpStore = EFS.getStore( newPath.toFile().toURI() );
		    		File tmpFile = tmpStore.toLocalFile( EFS.NONE, null );
		    		
		    		// Check whether the file is already in the temp dir
		    		// If it is, delete it
		    		if ( tmpFile.exists() && !tmpFile.canWrite() ) {
		    			tmpFile.delete();		    			
		    		}
		    		// Copy the file 
	    			srcFile.copy( tmpStore, EFS.OVERWRITE, null );
	    			
	    			// Add the original file into history
	    			CopyHistory.setCopyHistory( tmpFile, fileToOpen );
	    			
	    			fileToOpen = tmpFile;
	    			// Make it readonly
	    			fileToOpen.setReadOnly();	    			
	    		}
	    		catch ( CoreException e ) {
	    			return false;
	    		}
	    	}
	    	
	    	IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
	    	// Get id of ABL editor to use it for a new file
	    	String editorId = PlatformUI.getWorkbench().getEditorRegistry().getDefaultEditor( "x.cls" ).getId();
	    	// ITextEditor editor = ( ITextEditor ) 
	    	IDE.openEditor( page, fileToOpen.toURI( ), editorId, true );	    	

	    } catch ( PartInitException e ) {
	    	return false;	        
	    }		
	    return true;
	}
	
	/**
	 * Closes file opened in the active editor
	 * @return true if file has been closed OK
	 */
	public static boolean closeFileInCurrentEditor() {
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		return page.closeEditor(page.getActiveEditor(), false);		
	}
	
	/**
	 * Opens an external file in the editor as a link
	 * @param fileToOpen - file to open
	 * @param readonly - open file in readonly mode
	 * @return true - if file's been opened successfully
	 */
	public static boolean openFileInEditorAsLink(File fileToOpen, boolean readonly) {
		boolean result = true;

		try {
			IPath location = new Path(fileToOpen.getAbsolutePath());
			IFile file = lastProject.getProject().getFile(location.lastSegment());
			file.createLink(location, IResource.FILE, null);
			// if (readonly) file.setReadOnly(true);
			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			if (page != null) IDE.openEditor(page, file);
		}
		catch (Exception e) {
			result = false;	
		}		
		return result;		
	}
	
	/**
	 * Looks for a file using PROPATH
	 * @param project - OpenEdge project
	 * @param filename - a file to find
	 * @return file
	 */
	public static File lookForFileInPropath(OEProject project, String filename) {
		IPath file = project.getPropath().search(filename, false);
		if (file != null) 
			return file.toFile();
		else
			return new File(filename);
	}
	
	/**
	 * Gets file opened in the editor even if the file doesn't belong to the workspace
	 * @param window
	 * @return File
	 */
	public static File getEditorFile(IWorkbenchWindow window) {
		IPath path = OECommonServicesPlugin.getEditorPath();
		
		if (path == null) {
			IEditorPart editorPart = window.getActivePage().getActiveEditor();
			if(editorPart == null) return null;
			FileStoreEditorInput fileStore = (FileStoreEditorInput) editorPart.getEditorInput();
			return new File(fileStore.getURI());
		}
		return path.toFile();
	}
	

	/**
	 * Runs plugin action
	 * @param plugin
	 */
	public static void runPlugin(IKsvPlugin plugin) {
		// Get the current OE project
		OEProject project = getActiveOEProject();
		
		// If the current project isn't openedge's try the last openedge's one
		if (project == null) 
			project = KsvUtils.lastProject;

		if (project != null) {
			
			// Remember the openedge project
			KsvUtils.lastProject = project;
			
			plugin.runAction(project);
		}
	}
	
	/**
	 * Opens a file using OpenEdge PROPATH
	 * @param filename 		- file to open
	 * @param project  		- OpenEdge project whose PROPATH should be used
	 * @param showMessage	- show any message
	 * @param window		- current window
	 * @return
	 */
	public static boolean openFileInPropath (String filename, OEProject project, boolean showMessage,IWorkbenchWindow window) {
		boolean fileOpened = false;
		
		if ( filename.equals( "" ) ) return false;
		
		File fileToOpen = new File(filename);
		// If user has typed an unreal name, try to find it using PROPATH
		if (!fileToOpen.exists())
			fileToOpen = lookForFileInPropath (project, fileToOpen.getName());
		// If there's anything what can be opened, open it
		if (fileToOpen.exists())				
			fileOpened = openFileInEditor(fileToOpen );	
		else
		{
			if (showMessage) {
				MessageDialog.openInformation(
						window.getShell(), 
						"KsvPlugin", 
						"File [" + fileToOpen.getName() + "] cannot be found.");
			}
		}
		return fileOpened;
	}
	
	/**
	 * Add dirs into project's propath
	 * @param project 	- OE project
	 * @param pathList	- list of folder to add into PROPATH
	 * @param save		- make PROAPTH persistent
	 */
	public static void AddDirsToPropath ( OEProject project, String pathList, boolean save ) {
		
		ProjectPropath propath = project.getPropathHandler();
		String[] paths = pathList.split(",");		
				
		List<PropathEntry> propathEntries = new ArrayList<PropathEntry>();
		Collections.addAll( propathEntries, propath.getPropathEntries() );
		
		for( String path : paths ) {
			// Check before adding if the path is already in the list 
			boolean found = false;
			for (PropathEntry pEntry : propathEntries ) {
				if ( pEntry.getPath().equalsIgnoreCase( path ) ) {
					found = true;
					break;
				}				
			}
			// If path isn't in PROPATH
			if ( !found ) {
				PropathEntry entry = new PropathEntry( "dir" );		
				entry.setPath( path );		
				propathEntries.add( entry );
			}
		}
		// Change PROPATH and save it for the next runs
		propath.setPropathEntries( propathEntries.toArray( new PropathEntry[propathEntries.size()] ) );
		if ( save ) propath.savePropathFile();
	}
	
	/**
	 * Set new PROPATH for project
	 * @param project	- a OE project
	 * @param pathList	- list of folder for PROPATH
	 * @param save		- make PROAPTH persistent
	 */
	public static void SetPropath ( OEProject project, String pathList, boolean save ) {
		// Reset the current PROPATH		
		ProjectPropath propath = project.getPropathHandler();
		propath.setPropathEntries( null );
		// Set a new PROPATH
		AddDirsToPropath( project, pathList, save );				
	}
	
	public static OEProject getLastProject() {
		return lastProject;
	}
	

	static OEProject lastProject;	
	
}
